import { INumberIterator } from "../../src/param/param-value-iterator";
import { ParamValueMode } from "../../src/param/param-value-mode";
import { ParamValues } from "../../src/param/param-values";

function checkNumberList(it: INumberIterator, exp: number[]) {
    let n = 0;
    for (const v of it) {
        expect(v).toEqual(exp[n]);
        n++;
    }
    expect(n).toEqual(exp.length);
}

describe("paramvalues iterator : ", () => {
    it("vide (1)", () => {
        const p = new ParamValues();
        expect(() => p.getValuesIterator()).toThrow(
            new Error("ParamValueIterator : mode de génération de valeurs undefined incorrect")
        );
    });

    it("vide (2)", () => {
        const p = new ParamValues();
        p.valueMode = ParamValueMode.SINGLE;
        expect(() => p.getValuesIterator()).toThrow(new Error("ParamValues : valeur fixe non définie"));
    });

    it("vide (3)", () => {
        const p = new ParamValues();
        p.valueMode = ParamValueMode.MINMAX;
        expect(() => p.getValuesIterator()).toThrow(new Error("ParamValues : valeur min non définie"));
    });

    it("vide (4)", () => {
        const p = new ParamValues();
        p.valueMode = ParamValueMode.LISTE;
        expect(() => p.getValuesIterator()).toThrow(new Error("ParamValues : liste de valeurs non définie"));
    });

    it("single (1)", () => {
        const p = new ParamValues();
        p.setValues(0);
        const it = p.getValuesIterator();
        expect(it.hasNext).toBeTruthy();
        const n1 = it.next();
        expect(n1.value).toEqual(0);
        expect(n1.done).toBeFalsy();
        expect(it.hasNext).toBeFalsy();
        const n2 = it.next();
        expect(n2.value).toBeUndefined();
        expect(n2.done).toBeTruthy();
    });

    it("single (2)", () => {
        const p = new ParamValues();
        p.setValues(0);
        checkNumberList(p.getValuesIterator(), [0]);
    });

    it("minmax (1)", () => {
        const p = new ParamValues();
        p.setValues(0, 1, 2);
        checkNumberList(p.getValuesIterator(), [0]);
    });

    it("minmax (2)", () => {
        const p = new ParamValues();
        p.setValues(0, 1, 0.5);
        checkNumberList(p.getValuesIterator(), [0, 0.5, 1]);
    });

    it("liste (1)", () => {
        const p = new ParamValues();
        p.setValues([]);
        const it = p.getValuesIterator();
        expect(it.next().done).toBeTruthy();
        checkNumberList(p.getValuesIterator(), []);
    });

    it("liste (2)", () => {
        const p = new ParamValues();
        p.setValues([0]);
        checkNumberList(p.getValuesIterator(), [0]);
    });

    it("liste (3)", () => {
        const p = new ParamValues();
        p.setValues([1, 0.1]);
        checkNumberList(p.getValuesIterator(), [1, 0.1]);
    });
});
