
import { PabNombre, PabNombreParams } from "../../src/pab/pab_nombre";
import { checkResult } from "../test_func";

function pabNombreTest(varTest: string, expected: number) {
    describe("Calc(): ", () => {
        it(varTest + " should be " + expected, () => {
            const prms = new PabNombreParams(
                7.5,      // Chute totale DHT
                10,    // Nombre de bassins N
                0.75,    // Chute entre bassins DH
            );

            const nub = new PabNombre(prms);
            prms[varTest].v = undefined;

            checkResult(nub.Calc(varTest, 0), expected);
        });
    });
}

describe("Class PabNombre: ", () => {

    pabNombreTest("DHT", 7.5);
    pabNombreTest("N", 10);
    pabNombreTest("DH", 0.75);

    it ("DHR should be 0.3", () => {
        const prms = new PabNombreParams(
            7.5,      // Chute totale DHT
            666,    // Nombre de bassins N
            0.8,    // Chute entre bassins DH
        );

        const nub = new PabNombre(prms);

        nub.Calc("N", 0);
        expect(nub.result.vCalc).toBe(9);
        expect(nub.result.getExtraResult("DHR")).toBeCloseTo(0.3);
    });

    it ("DHR should be 0", () => {
        const prms = new PabNombreParams(
            3,      // Chute totale DHT
            666,    // Nombre de bassins N
            0.2,    // Chute entre bassins DH
        );

        const nub = new PabNombre(prms);

        nub.Calc("N", 0);
        expect(nub.result.vCalc).toBe(15);
        expect(nub.result.getExtraResult("DHR")).toBe(0);
    });

    it ("non-integer number of basins should lead to log error entry", () => {
        const prms = new PabNombreParams(
            3,      // Chute totale DHT
            4.5,    // Nombre de bassins N
            0.2,    // Chute entre bassins DH
        );

        const nub = new PabNombre(prms);

        expect(nub.Calc("DH", 0).log.messages.length).toBeGreaterThan(0);
    });
});
