/**
 * IMPORTANT !
 * Décommenter temporairement la ligne suivante (import { } from "./mock_jasmine")
 * Pour exécuter ce code dans le débugger.
 * Faire de même avec le fichier test_func.ts
 */
// import { describe, expect, it, xdescribe, xit } from "../mock_jasmine";

import { RegimeUniforme } from "../../src/regime_uniforme";
import { cSnRectang, ParamsSectionRectang } from "../../src/section/section_rectang";
import { Result } from "../../src/util/result";
import { precDist } from "../test_config";
import { checkResult, equalEpsilon } from "../test_func";

describe("Class RegimeUniforme / section rectangulaire :", () => {
    describe("pas de débordement : ", () => {
        /**
         * test de la largeur de fond (= largeur de berge pour le rectangulaire)
         */
        it("LargeurBerge should be 2.5", () => {
            const prms = new ParamsSectionRectang(0.8, // tirant d'eau
                undefined, // largeur de fond
                40, //  Ks=Strickler
                1.568, // Q=Débit
                0.001, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;
            const sect = new cSnRectang(prms, false);
            const ru = new RegimeUniforme(sect, false);

            checkResult(ru.Calc("LargeurBerge", 0), 2.5);
        });

        it("Strickler should be 30.619", () => {
            const prms = new ParamsSectionRectang(0.8, // tirant d'eau
                2.5, // largeur de fond
                undefined, //  Ks=Strickler
                1.2, // Q=Débit
                0.001, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Ks", 1e-8), 30.619);
        });

        it("If should be 0.001", () => {
            const prms = new ParamsSectionRectang(0.8, // tirant d'eau
                2.5, // largeur de fond
                40, //  Ks=Strickler
                1.568, // Q=Débit
                undefined, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect, false);

            // nom variable à calculer, valeur de Ks
            checkResult(ru.Calc("If", 0.001), 0.001);
        });

        it("Q should be 1.568", () => {
            const prms = new ParamsSectionRectang(0.8, // tirant d'eau
                2.5, // largeur de fond
                40, //  Ks=Strickler
                undefined, // Q=Débit
                0.001, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Q", 0), 1.568);
        });

        it("Q should be 0.731", () => {
            const prms = new ParamsSectionRectang(1.1, // tirant d'eau
                3, // largeur de fond
                30, //  Ks=Strickler
                undefined, // Q=Débit
                0.0001, // If=pente du fond
                1.2, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Q", 0), 0.731);
        });

        it("Y should be 0.663", () => {
            const prms = new ParamsSectionRectang(undefined, // tirant d'eau
                2.5, // largeur de fond
                40, //  Ks=Strickler
                1.2, // Q=Débit
                0.001, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            // nom variable à calculer, valeur de Ks
            checkResult(ru.Calc("Y", 0), 0.663);
        });
    });

    describe("débordement : ", () => {
        /**
         * test de la largeur de fond (= largeur de berge pour le rectangulaire)
         */
        it("LargeurBerge should be 2.5", () => {
            const prms = new ParamsSectionRectang(2, // tirant d'eau
                undefined, // largeur de fond
                40, //  Ks=Strickler
                5.31, // Q=Débit
                0.001, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms, false);
            const ru = new RegimeUniforme(sect, false);

            checkResult(ru.Calc("LargeurBerge", 0), 2.5);
        });

        it("Strickler should be 9.04", () => {
            const prms = new ParamsSectionRectang(2, // tirant d'eau
                2.5, // largeur de fond
                undefined, //  Ks=Strickler
                1.2, // Q=Débit
                0.001, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Ks", 1e-8), 9.04);
        });

        it("If should be 0.001", () => {
            const prms = new ParamsSectionRectang(2, // tirant d'eau
                2.5, // largeur de fond
                9.04, //  Ks=Strickler
                1.2, // Q=Débit
                undefined, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect, false);

            // nom variable à calculer, valeur de Ks
            checkResult(ru.Calc("If", 0.001), 0.001);
        });

        it("Q should be 5.31", () => {
            const prms = new ParamsSectionRectang(2, // tirant d'eau
                2.5, // largeur de fond
                40, //  Ks=Strickler
                undefined, // Q=Débit
                0.001, // If=pente du fond
                1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Q", 0), 5.31);
        });

        it("Q should be 1.624", () => {
            const prms = new ParamsSectionRectang(2, // tirant d'eau
                3, // largeur de fond
                30, //  Ks=Strickler
                undefined, // Q=Débit
                0.0001, // If=pente du fond
                1.2, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            checkResult(ru.Calc("Q", 0), 1.624);
        });

        it("Y should be 0.663", () => {
            const prms = new ParamsSectionRectang(undefined, // tirant d'eau
                2.5, // largeur de fond
                40, //  Ks=Strickler
                1.2, // Q=Débit
                0.001, // If=pente du fond
                0.1, // YB=hauteur de berge
            );
            prms.Pr.v = precDist;

            const sect = new cSnRectang(prms);
            const ru = new RegimeUniforme(sect);

            // nom variable à calculer, valeur de Ks
            checkResult(ru.Calc("Y", 0), 0.663);
        });
    });
});
