import { CalculatorType } from "../compute-node";
import { Nub } from "../nub";
import { ParamCalculability, ParamDefinition, ParamFamily } from "../param/param-definition";
import { ParamDomainValue } from "../param/param-domain";
import { ParamsEquation } from "../param/params-equation";
import { Result } from "../util/result";

export class PabDimensionParams extends ParamsEquation {
    [key: string]: any; // pour pouvoir faire this['methode]();

    /** Longueur L */
    private _L: ParamDefinition;

    /** Largeur W */
    private _W: ParamDefinition;

    /** Tirant d'eau Y */
    private _Y: ParamDefinition;

    /** Volume V */
    private _V: ParamDefinition;

    constructor(rL: number, rW: number, rY: number, rV?: number) {
        super();
        this._L = new ParamDefinition(this, "L", ParamDomainValue.POS, rL, ParamFamily.LENGTHS);
        this._W = new ParamDefinition(this, "W", ParamDomainValue.POS, rW, ParamFamily.WIDTHS);
        this._Y = new ParamDefinition(this, "Y", ParamDomainValue.POS, rY, ParamFamily.HEIGHTS);
        this._V = new ParamDefinition(this, "V", ParamDomainValue.POS, rV, ParamFamily.VOLUMES);

        this.addParamDefinition(this._L);
        this.addParamDefinition(this._W);
        this.addParamDefinition(this._Y);
        this.addParamDefinition(this._V);
    }

    get L() {
        return this._L;
    }

    get W() {
        return this._W;
    }

    get Y() {
        return this._Y;
    }

    get V() {
        return this._V;
    }
}

// tslint:disable-next-line:max-classes-per-file
export class PabDimension extends Nub {
    constructor(prms: PabDimensionParams, dbg: boolean = false) {
        super(prms, dbg);
        this._calcType = CalculatorType.PabDimensions;
    }

    /**
     * paramètres castés au bon type
     */
    get prms(): PabDimensionParams {
        return this._prms as PabDimensionParams;
    }

    public Equation(sVarCalc: string): Result {
        let v: number;

        switch (sVarCalc) {
            case "V":
                v = this.prms.L.v * this.prms.W.v * this.prms.Y.v;
                break;

            case "L":
                v = this.prms.V.v / this.prms.W.v / this.prms.Y.v;
                break;

            case "W":
                v = this.prms.V.v / this.prms.L.v / this.prms.Y.v;
                break;

            case "Y":
                v = this.prms.V.v / this.prms.L.v / this.prms.W.v;
                break;

            default:
                throw new Error("PabDimension.Equation() : invalid variable name " + sVarCalc);
        }

        return new Result(v, this);
    }

    /**
     * paramétrage de la calculabilité des paramètres
     */
    protected setParametersCalculability() {
        this.prms.L.calculability = ParamCalculability.EQUATION;
        this.prms.W.calculability = ParamCalculability.EQUATION;
        this.prms.Y.calculability = ParamCalculability.EQUATION;
        this.prms.V.calculability = ParamCalculability.EQUATION;
    }

}
