export enum MessageCode {
    /**
     * pas de pb !
     */
    ERROR_OK,

    /**
     * la dichotomie n'a pas pu trouver automatiquement d'intervalle de départ
     * car la valeur initiale de la variable est trop haute
     */
    ERROR_DICHO_INITVALUE_HIGH,

    /**
     * la dichotomie n'a pas pu trouver automatiquement d'intervalle de départ
     * car la valeur initiale de la variable est trop basse
     */
    ERROR_DICHO_INITVALUE_LOW,

    /**
     * la dichotomie n'a pas pu trouver automatiquement d'intervalle de départ
     * car la valeur cible de la fonction n'existe pas pour des valeurs de la
     * variable dans son domaine de définition, cad il n'existe pas de solution
     */
    ERROR_DICHO_INIT_DOMAIN,

    /**
     * la dichotomie n'a pas pu converger
     */
    ERROR_DICHO_CONVERGE,

    /**
     * le pas pour la recherche de l'intervalle de départ est =0
     */
    ERROR_DICHO_NULL_STEP,

    /**
     * l'augmentation du pas pour la recherche de l'intervalle de départ est incorrecte (=0)
     */
    ERROR_DICHO_INVALID_STEP_GROWTH,

    /**
     * impossible de déterminer le sens de variation de la fonction
     */
    ERROR_DICHO_FUNCTION_VARIATION,

    /**
     * la cote amont Z1 est plus basse que la cote aval Z2
     */
    ERROR_ELEVATION_ZI_LOWER_THAN_Z2,

    /**
     * les bornes de l'intervalle d'un ParamDomain sont incorrectes
     */
    ERROR_PARAMDOMAIN_INTERVAL_BOUNDS,

    /**
     * la valeur du ParamDomain est incorrecte
     */
    ERROR_PARAMDOMAIN_INVALID,

    /**
     * la calculabilité d'un ParamDefinition est non définie
     */
    ERROR_PARAMDEF_CALC_UNDEFINED,

    /**
     * la valeur d'un ParamDefinition est non définie
     */
    ERROR_PARAMDEF_VALUE_UNDEFINED,

    /**
     * la valeur de la cible d'un ParamDefinition est non définie
     */
    ERROR_PARAMDEF_LINKED_VALUE_UNDEFINED,

    /**
     * la valeur d'un ParamDefinition ne peut pas être changée
     */
    ERROR_PARAMDEF_VALUE_FIXED,

    /**
     * la valeur d'un ParamDefinition doit être entière
     */
    ERROR_PARAMDEF_VALUE_INTEGER,

    /**
     * la valeur d'un ParamDefinition doit être > 0
     */
    ERROR_PARAMDEF_VALUE_POS,

    /**
     * la valeur d'un ParamDefinition doit être >= 0
     */
    ERROR_PARAMDEF_VALUE_POSNULL,

    /**
     * la valeur d'un ParamDefinition doit être = 0
     */
    ERROR_PARAMDEF_VALUE_NULL,

    /**
     * la valeur d'un ParamDefinition est en dehors de son intervalle autorisé
     */
    ERROR_PARAMDEF_VALUE_INTERVAL,

    /**
     * la valeur passée à une méthode de la classe Interval est undefined
     */
    ERROR_INTERVAL_UNDEF,

    /**
     * la valeur passée à une méthode de la classe Interval est hors de l'intervalle défini
     */
    ERROR_INTERVAL_OUTSIDE,

    /**
     * internationalisation : langue non prise en charge
     */
    ERROR_LANG_UNSUPPORTED,

    /**
     * courbes de remous : Arrêt du calcul : hauteur critique atteinte à l'abscisse x
     */
    WARNING_REMOUS_ARRET_CRITIQUE,

    /**
     * courbe de remous : Condition limite aval >= Hauteur critique : calcul de la partie fluviale à partir de l'aval
     */
    INFO_REMOUS_CALCUL_FLUVIAL,

    /**
     * courbe de remous : Condition limite amont <= Hauteur critique :
     * calcul de la partie torrentielle à partir de l'amont
     */
    INFO_REMOUS_CALCUL_TORRENTIEL,

    /**
     * courbe de remous : ressaut hydraulique détecté à l'amont/aval de l'abscisse x
     */
    INFO_REMOUS_RESSAUT_DEHORS,

    /**
     * courbe de remous : Largeur au niveau des berges
     */
    INFO_REMOUS_LARGEUR_BERGE,

    /**
     * courbe de remous : Tirant d'eau critique
     */
    INFO_REMOUS_H_CRITIQUE,

    /**
     * courbe de remous : Tirant d'eau normal
     */
    INFO_REMOUS_H_NORMALE,

    /**
     * courbe de remous : Ressaut hydraulique détecté entre les abscisses Xmin et Xmax m
     */
    INFO_REMOUS_RESSAUT_HYDRO,

    /**
     * courbe de remous : La pente de la ligne d'eau est trop forte à l'abscisse x m
     */
    ERROR_REMOUS_PENTE_FORTE,

    /**
     * courbe de remous : Condition limite aval < Hauteur critique : pas de calcul possible depuis l'aval
     */
    ERROR_REMOUS_PAS_CALCUL_DEPUIS_AVAL,

    /**
     * courbe de remous : Condition limite amont > Hauteur critique : pas de calcul possible depuis l'amont
     */
    ERROR_REMOUS_PAS_CALCUL_DEPUIS_AMONT,

    /**
     * section : Non convergence du calcul de la hauteur critique (Méthode de Newton)
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HCRITIQUE,

    /**
     * section : Non convergence du calcul de la hauteur normale (Méthode de Newton)
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HNORMALE,

    /**
     * section : Non convergence du calcul de la hauteur conjuguée (Méthode de Newton)
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HCONJUG,

    /**
     * section : Non convergence du calcul de la hauteur correspondante (Méthode de Newton)
     * pour le calcul de la hauteur fluviale
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HFLU,

    /**
     * section : Non convergence du calcul de la hauteur correspondante (Méthode de Newton)
     * pour le calcul de la hauteur torrentielle
     */
    ERROR_SECTION_NON_CONVERGENCE_NEWTON_HTOR,

    /**
     * section : La pente est négative ou nulle, la hauteur normale est infinie
     */
    ERROR_SECTION_PENTE_NEG_NULLE_HNORMALE_INF,

    /**
     * section : calcul impossible à cause d'un périmètre nul
     */
    ERROR_SECTION_PERIMETRE_NUL,

    /**
     * section : calcul impossible à cause d'un rayon nul
     */
    ERROR_SECTION_RAYON_NUL,

    /**
     * section : calcul impossible à cause d'une surface nulle
     */
    ERROR_SECTION_SURFACE_NULLE,

    /**
     * newton : pas de convergence
     */
    ERROR_NEWTON_NON_CONVERGENCE,

    /**
     * newton : dérivée nulle
     */
    ERROR_NEWTON_DERIVEE_NULLE,

    /**
     * Le paramètre "Cote de radier" ne peut pas être calculé avec cette loi de débit
     */
    ERROR_STRUCTURE_ZDV_PAS_CALCULABLE,

    /**
     * Le débit passant par les autres ouvrages est trop important
     */
    ERROR_STRUCTURE_Q_TROP_ELEVE,

    /**
     * StructureKivi : La pelle du seuil doit mesurer au moins 0,1 m. Le coefficient béta est forcé à 0.
     */
    WARNING_STRUCTUREKIVI_PELLE_TROP_FAIBLE,

    /**
     * StructureKivi : h/p ne doit pas être supérieur à 2,5. h/p est forcé à 2,5.
     */
    WARNING_STRUCTUREKIVI_HP_TROP_ELEVE,
}

/**
 * niveau de criticité du message
 */
export enum MessageSeverity {
    ERROR, WARNING, INFO
}

/**
 * Résultat de calcul comprenant la valeur du résultat et des calculs annexes (flag, calculs intermédiaires...)
 */
export class Message {

    /** Variables intermédiaires, flags d'erreur */
    public extraVar: { [key: string]: any };

    /**
     * code du message
     */
    private _code: MessageCode;

    constructor(c: MessageCode) {
        this._code = c;
        this.extraVar = {};
    }

    get code() { return this._code; }

    /**
     * retourne le niveau de criticité (erreur, warning, info) du message
     */
    public getSeverity(): MessageSeverity {
        const m: string = MessageCode[this._code];
        const prefix: string = m.split("_")[0];
        for (const s in MessageSeverity) { // pas nécessaire, mais permet d'avoir un contrôle de validité
            if (MessageSeverity[s] === prefix) {
                // tslint:disable-next-line:no-eval
                return eval("MessageSeverity." + prefix);
            }
        }
        throw new Error("Message.getSeverity() : valeur de code '" + this._code + "' invalide");
    }

    public toString(): string {
        return MessageCode[this._code] + " " + JSON.stringify(this.extraVar);
    }
}
